package com.fossgalaxy.games.tbs.parameters;

import com.fossgalaxy.games.tbs.entity.Entity;

import java.util.HashMap;
import java.util.Map;

public class TerrainType {

    private final String id;
    public final Map<String, Integer> requiredTags;

    public transient Map<String, Integer> computedTags = new HashMap<>();
    public final String image;

    public TerrainType(String id){
        this.id = id;
        this.requiredTags = new HashMap<>();
        this.computedTags = new HashMap<>();
        this.image = null;
    }

    public TerrainType(TerrainType tt){
        this.id = tt.id;
        this.requiredTags = new HashMap<>(tt.requiredTags);
        this.computedTags = new HashMap<>();
        if(tt.computedTags != null){
            this.computedTags.putAll(tt.computedTags);
        }
        this.image = tt.image;
    }


    public String getID() {
        return id;
    }

    public String getImage() {
        return image;
    }

    public boolean isPassible(Entity entity) {
        if (requiredTags == null || requiredTags.isEmpty()){
            return false;
        }

        if(computedTags == null || computedTags.isEmpty()){
            computeTags();
        }

        for ( Map.Entry<String, Integer> tagItr : computedTags.entrySet() ) {

            String tag = tagItr.getKey();
            Integer terrainVal = tagItr.getValue();

            int entityVal = entity.getProperty(tag);

            if (entityVal >= terrainVal) {
                return true;
            }

        }

        return false;
    }

    private void computeTags(){
        computedTags = new HashMap<>();
        for(Map.Entry<String, Integer> tagItr : requiredTags.entrySet()){
            computedTags.put("ter-" + tagItr.getKey(), tagItr.getValue());
        }
    }

    public boolean isPassible(EntityType entityType) {
        if (requiredTags == null || entityType == null){
            return false;
        }

        if(computedTags == null || computedTags.isEmpty()){
            computeTags();
        }

        for ( Map.Entry<String, Integer> tagItr : computedTags.entrySet() ) {

            String tag = tagItr.getKey();
            Integer terrainVal = tagItr.getValue();

            int entityTypeVal = entityType.getProperty( tag, 0);

            if (entityTypeVal >= terrainVal) {
                return true;
            }

        }

        return false;
    }

    void setTag(String tag, int minLevel) {
        requiredTags.put(String.format("ter-%s", tag), minLevel);
    }

    public String getName() {
        return id;
    }

    public String toString() {
        return getName();
    }

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((requiredTags == null) ? 0 : requiredTags.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		TerrainType other = (TerrainType) obj;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (requiredTags == null) {
			if (other.requiredTags != null)
				return false;
		} else if (!requiredTags.equals(other.requiredTags))
			return false;
		return true;
	}
    
}
